package com.sun.org.apache.xml.internal.resolver;

import java.util.concurrent.ConcurrentHashMap;
import java.util.Vector;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

public class CatalogEntry
{
    protected static AtomicInteger nextEntry;
    protected static final Map<String, Integer> entryTypes;
    static final Vector entryArgs;
    protected int entryType;
    protected Vector args;
    
    static int addEntryType(String name, int numArgs) {
        int index = CatalogEntry.nextEntry.getAndIncrement();
        CatalogEntry.entryTypes.put(name, Integer.valueOf(index));
        CatalogEntry.entryArgs.add(index, Integer.valueOf(numArgs));
        return index;
    }
    
    public static int getEntryType(String name) throws CatalogException {
        if (!CatalogEntry.entryTypes.containsKey(name)) {
            throw new CatalogException(3);
        }
        Integer iType = (Integer)CatalogEntry.entryTypes.get(name);
        if (iType == null) {
            throw new CatalogException(3);
        }
        return (int)iType;
    }
    
    public static int getEntryArgCount(String name) throws CatalogException {
        return getEntryArgCount(getEntryType(name));
    }
    
    public static int getEntryArgCount(int type) throws CatalogException {
        try {
            Integer iArgs = (Integer)CatalogEntry.entryArgs.get(type);
            return (int)iArgs;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            throw new CatalogException(3);
        }
    }
    
    public CatalogEntry() {
        this.entryType = 0;
        this.args = null;
    }
    
    public CatalogEntry(String name, Vector args) throws CatalogException {
        this.entryType = 0;
        this.args = null;
        Integer iType = (Integer)CatalogEntry.entryTypes.get(name);
        if (iType == null) {
            throw new CatalogException(3);
        }
        int type = (int)iType;
        try {
            Integer iArgs = (Integer)CatalogEntry.entryArgs.get(type);
            if ((int)iArgs != args.size()) {
                throw new CatalogException(2);
            }
        }
        catch (ArrayIndexOutOfBoundsException e) {
            throw new CatalogException(3);
        }
        this.entryType = type;
        this.args = args;
        return;
    }
    
    public CatalogEntry(int type, Vector args) throws CatalogException {
        this.entryType = 0;
        this.args = null;
        try {
            Integer iArgs = (Integer)CatalogEntry.entryArgs.get(type);
            if ((int)iArgs != args.size()) {
                throw new CatalogException(2);
            }
        }
        catch (ArrayIndexOutOfBoundsException e) {
            throw new CatalogException(3);
        }
        this.entryType = type;
        this.args = args;
    }
    
    public int getEntryType() {
        return this.entryType;
    }
    
    public String getEntryArg(int argNum) {
        try {
            String arg = (String)this.args.get(argNum);
            return arg;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            return null;
        }
    }
    
    public void setEntryArg(int argNum, String newspec) throws ArrayIndexOutOfBoundsException {
        this.args.set(argNum, newspec);
    }
    
    static {
        CatalogEntry.nextEntry = new AtomicInteger(0);
        entryTypes = new ConcurrentHashMap();
        entryArgs = new Vector();
    }
}
